# -*- coding: utf-8 -*-
"""
Created on Thu Jun 27 17:03:14 2019

PREREQUISITES:
    The CSV files that are exported from the analyzer tool must use ; as (field) separator and , as decimal separatator
    The exported files must include following signals:
        Generator_cold_air_1-4
        Generator_warm_air_1 and 2
        Power
        Setpoint_active_power
        

NOTES:
    The values in the exported CSV are stored in reversed chronological order, 
    meaning that the last valid value is placed first. 
    If the file is inspected in Excel it must then be edited to include the separator
    
SPECIFICS:
    This script is designed to handle the four cold air values
    combined with the two hot air measurements, as well as the power
    for each of the generators
"""
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import time
import pandas as pd
from sklearn.linear_model import LinearRegression
import datetime as dt
import warnings
from IPython import get_ipython
get_ipython().run_line_magic('matplotlib', 'qt')       #Shows plots in separate window
#get_ipython().run_line_magic('matplotlib', 'inline')  #Gives inline plots

#Cold and warm air
#homePath = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Exported data/Scenario 1/Unit 1/'
homePath = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Exported data/Scenario 1/Unit 2/'

#start time
t1 = time.perf_counter()
#print('Processing started '+ time.asctime())

##############################import data######################################
###########################     UNIT 1
#filelist = ['Unit 1 scenario 1_25032017', 'Unit 1 scenario 1_31032017', # 'Unit 1 scenario 1_05052017',
#            'Unit 1 scenario 1_28062017', 'Unit 1 scenario 1_13102017', 'Unit 1 scenario 1_11122017',
#            'Unit 1 scenario 1_28042018', 'Unit 1 scenario 1_25082018',
#            'Unit 1 scenario 1_27022019','Unit 1 scenario 1_08062019']#Dynamic list of input files

###########################     UNIT 2
filelist = ['Unit 2 scenario 1_14052017','Unit 2 scenario 1_09072017',
            'Unit 2 scenario 1_04102017','Unit 2 scenario 1_27022018',
            'Unit 2 scenario 1_27062018','Unit 2 scenario 1_28022019',
            'Unit 2 scenario 1_27042019','Unit 2 scenario 1_23062019']

appended_data_dates = []                                                                    #Empty list to store list of dataframes

for i in range(len(filelist)):    
    filetype = '.csv'
    
    path = homePath + filelist[i] + filetype         #Seprated into an extra variable to allow re-use of filename
    print('\n'+filelist[i])
    
    df = pd.read_csv(path, sep=';', skiprows=1)   #Skips first row, because of meta- data(separator)
    
    df = df.iloc[::-1].reset_index(drop=True)     #Reverses the order of the data to make it chronological and resets the index accordingly
    
    ####  Resampling and combining measurements in one Data Frame  ################ 
    """
    The method creates a large resampled dataframe that consists of the 
    timestamps and measurements for each signal/measurement in the exported CSV-file
    
    Prerequisites
    ----------
    CSV-file with one, or more, measurements.
    The CSV must be separated by a (;), and the delimitor must be(,).
    
    Parameters
    ----------
    Dataframe containing imported data.
    
    Returns
    ----------
    Pandas Dataframe
    """
    
    noColumns = len(df.columns)                                                           #Finds the number of columns in the imported data stored in df
    noMeasurements = int(noColumns/7)                                                     #Finds the number of measurements. Each measurement contains 7 columns 
    
    appended_data = []                                                                    #Empty list to store list of dataframes
    
    j=0                                                                                   #Counter for column access
    #noMeasurements=1#FJERNES
    for i in range(noMeasurements):
        datetime=pd.to_datetime(df.iloc[:,j]).dropna()                                    #Collects timestamp and converts it to python datetime-format and removes NaTs
    #    datetime=pd.to_datetime(df.iloc[:,0]).dropna()#Fjernes
    #    print('datetime')
    #    print(datetime)
        measure_nan = df.iloc[:,j+5].dropna()                                            #Drops NaNs from the data frame
    #    measure_nan = df.iloc[:,5].dropna()#Fjernes 
    #    print('measure_nan')
    #    print(measure_nan)
        measurement = measure_nan.apply(lambda x: x.replace(',', '.')).astype('float')    #Collects measurement, replaces , with ., in order to be able to convert it to a float
    #    print('measurement')
    #    print(measurement)
        ts = pd.concat([datetime, measurement], axis=1)                                   #Concatenate corresponding timestamp and measurement
    #    print('ts')
    #    print(ts)
        remove_duplicates = ts.drop_duplicates(subset=['Date ' + str(i+1)], keep='first') #Drops value when there are duplicate timestamps. Keeps the first value
    #    print('remove_duplicates')
    #    print(remove_duplicates)
    #    remove_duplicates.drop(remove_duplicates.head(2).index,inplace=True)              #The first TWO values often have a very large gap, to ensure that it is not given too much weight, it is dropped
    #    remove_duplicates.drop(remove_duplicates.tail(1).index,inplace=True)              #The last values often has a very large gap, to ensure that it is not given too much weight, it is dropped
        reindex = remove_duplicates.set_index('Date ' + str(i+1))                         #Re-index to get datetime as index, necessary as index for resampling-method
    #    print('reindex')
    #    print(reindex)
        #S is seconds, T is minute. '3T' is 3 minutes, H is hour, D is day
    #    resample = reindex.resample('S').pad()                                           #Second-sampling with forwardfill(resample)
        resample = reindex.resample('T').pad()                                           #Minute-sampling with forwardfill(resample)
    #    resample = reindex.resample('H').pad()                                           #Hour-sampling with forwardfill(resample)
    #    resample = reindex.resample('D').pad()                                           #Day-sampling with forwardfill(resample)
        resample_drop_first_row = resample.drop(resample.index[[0]])                      #If the value provided to the resampling method is second-sampled it will give a NaN on the first value, and be reflected on the second, now minute-sampled value
        df_index = resample_drop_first_row.reset_index()                                  #Reindexes the dataframe to get it zero-indexed. This is necessary in order to have the same index for all dataframes, which is a requirement for appending
    #    print('df_index')
    #    print(df_index)
        appended_data.append(df_index)                                                    #Appends the dataframe to a list of dataframes
        j+=7                                                                              #Increments the counter with the number of colums per measurement, to gather all the measurements
    appended_data = pd.concat(appended_data, axis=1)                                      #Concatenate the dataframes to one large dataframe
    resampled_df = appended_data                                                          #Renames the large dataframe by assigning it to a new variable that better explains how the data has been preprocessed
    #print(resampled_df)
    
    
    #### Method for finding the first common timestamp in a dynamic dataframe #####
    noColumns_complete = len(resampled_df.columns)                               #Finds the number of columns in the imported data stored in df
    noMeasurements_complete = int(noColumns_complete/2)                                   #Finds the number of measurements. Each measurement(now) contains 2 columns; timestamp and measurement 
    
    ###Method below finds the first common date in the first row, ergo the last starting date
    k=0
    dt1 = resampled_df.iloc[0,0]                                                          #First timestamp in first column
    for i in range(noMeasurements_complete-1):                                            #Does the operation for every measurement
        dt2 = resampled_df.iloc[0, (k+2)]                                                 #Second timestamp, located in the third column
        if dt1 < dt2:                                                                     #Compares the first and last timestamp to find the largest(most recent)
            mostRecent = dt2                                                              #If dt2 is the largest, it is assigned to the mostRecent variable
            dt1 = dt2                                                                     #dt2 is also assigned to the dt1 vairable to compare this value to the timestamp of the next measurement
        else:                                                                             #if dt1 is more recent than dt2, the value of dt1 is kept as mostRecent timestamp
            mostRecent = dt1
        k += 2                                                                            #Iterate the K-variable to go to next datetime-column
    print('Most recent common date: '+ str(mostRecent))
        
    ##Method below crops the dataframe 
    cropped_data = []                                                                                           #Empty list to store cropped dataframes
    m=0
    for l in range(noMeasurements_complete):                                                                    # Will perform an iteration for every measurement
        res_index = next((i for i, j in enumerate(resampled_df.iloc[:, m]) if j == mostRecent), None)  #"Search" each column to find the index for the datetime stored in mostRecent-variable from previous method 
    #Method above is inspired by SilenGhost's reply on Stackoverflow, last visited 12.06.19(https://stackoverflow.com/questions/3229626/python-finding-index-of-first-non-empty-item-in-a-list/3229644)
        datetime_crop = resampled_df.iloc[res_index:, m]                                               # Crops every datetime-column from the top down to the index provided by previous value which is the most recent common date
        measurement_crop = resampled_df.iloc[res_index:, m+1]                                          # Crops every measurement-column from the top down to the index provided by previous value
        ts_crop = pd.concat([datetime_crop, measurement_crop], axis=1)                                          #Concatenates the cropped datetime and measurement columns to a timeseries
        ts_crop = ts_crop.reset_index(drop=True)                                                                #Resets the index for the timeseries, since it has been cropped top-down, since last index reset
        cropped_data.append(ts_crop)                                                                            #Appends the cropped dataframe to a list of dataframes(together with the ones already cropped)
        m+=2                                                                                                    #Increments the counter with the number of colums per measurement, to gather all the measurements
    cropped_data = pd.concat(cropped_data, axis=1)                                                              #Concatenate the dataframes to one large dataframe
    cropped_resampled_data = cropped_data.dropna()                                                                 #Drops any row that contains a NaN(at end of the complete dataframe(due to uneven number of samples for each measurement/uneven length of timeseries))
    
    #print(cropped_resampled_data)
    
    ################# Method for removing duplicate timestap columns #############
    reindexed_cropped_resampled_data = cropped_resampled_data.set_index(cropped_resampled_data.columns[0])                     #Set one common index for all measurement values
    reindexed_cropped_resampled_data.index.names = ['Timestamp']                                                     #Renames the common index column
    
    #Finds the number of columns and timestamps for drop of dynamic number of columns
    noColumns_prd = len(reindexed_cropped_resampled_data.columns)                                                    #Finds the number of columns in the imported data stored in df
    noTimestamps_prd = int(noColumns_prd/2)                                                                     #Finds the number of measurements
      
    #o = noTimestamps_prd+1                                                                                     #For addressing coloumn name in method below
    o = noColumns_prd-2                                                                                         #For addressing coloumn index in method below
    for n in range(noTimestamps_prd ):
    #    preprocessed_reindexed_data.drop(columns=["Date " + str(o)], axis = 1, inplace=True)                   #For addressing coloumn name 
        reindexed_cropped_resampled_data.drop(reindexed_cropped_resampled_data.columns[o], axis = 1, inplace=True)        #For addressing coloumn index
    #    o -= 1                                                                                                 #For addressing coloumn name in method above
        o -= 2   
                                                                                                   #For addressing coloumn index in method below
    df_preprocessed = reindexed_cropped_resampled_data #The preprocessed dataframe, renamed to emphasize that this is the finished dataframe
    #df_preprocessed.to_excel(filename + 'seconds_preprocessed.xlsx')                               #Writes the (often very) large dataframe to an Excel-file(Very time consuming)
    #print(df_preprocessed)
    #print(df_preprocessed.dtypes)
    
    ########################Analysis ##############################################
    
    #Lage ein gjennomsnitt av alle verdier på ein rad
    
    df_preprocessed['Mean cold air'] = df_preprocessed.iloc[:,0:4].mean(axis=1)
    df_preprocessed['Mean warm air'] = df_preprocessed.iloc[:,4:6].mean(axis=1)
    df_preprocessed['Diff. mean'] = df_preprocessed.iloc[:,9]-df_preprocessed.iloc[:,8]
    df_preprocessed['Cooler 1'] = df_preprocessed.iloc[:,9]-df_preprocessed.iloc[:,0]
    df_preprocessed['Cooler 2'] = df_preprocessed.iloc[:,9]-df_preprocessed.iloc[:,1]
    df_preprocessed['Cooler 3'] = df_preprocessed.iloc[:,9]-df_preprocessed.iloc[:,2]
    df_preprocessed['Cooler 4'] = df_preprocessed.iloc[:,9]-df_preprocessed.iloc[:,3]
    df_preprocessed['Mean, coolers'] = df_preprocessed.iloc[:,11:15].mean(axis=1)
#    print(df_preprocessed.iloc[:,7])
#    print(df_preprocessed.iloc[:,8])
#    print(df_preprocessed.iloc[:,9])
#    print(df_preprocessed.iloc[:,16])
    
#    print(df_preprocessed.dtypes)
    print('First valid index: '+ str(df_preprocessed.first_valid_index()))
    print('Last valid index: '+ str(df_preprocessed.last_valid_index()))
    
    #print(df_preprocessed[['Mean_cold', 'Mean_warm', 'Mean_diff']])
    #print(df_preprocessed[['Cooler 1', 'Cooler 2', 'Cooler 3', 'Cooler 4']])
    
    
    #End here
#    appended_data_dates = [] 
    appended_data_dates.append(df_preprocessed)
scenario_1 = pd.concat(appended_data_dates, axis=0)                                                              #Concatenate the dataframes to one large dataframe

#print(scenario_1.dtypes) 
#scenario_1.to_excel('Scenario_1.xlsx')   # Output to Excel-file

#######################################  Scatter plot  ##############################
plt.rcParams.update({'font.size': 20})
#fig, sctr = plt.subplots()
#sctr2 = sctr.twinx()

x1 = scenario_1.index
y1 = scenario_1['Mean, coolers']
#y1 = scenario_1['Cooler 1']
#y1 = scenario_1['Cooler 2']
#y1 = scenario_1['Cooler 3']
#y1 = scenario_1['Cooler 4']
y2 = scenario_1['Measurement 7']

#plt.set_ylabel('Temperature [K]')
plt.ylabel(r'$\Delta$ Temperature[K]')
plt.xlabel('Time')

plt.plot([],[]) #Has to be present to "correct" the plot

sctr = plt.scatter(x1, y1, c = y2, cmap='gist_rainbow')
plt.colorbar(sctr, format='%d MW')
plt.clim(36.5,41.5) #Limits for colorbar UNIT 1

##Regression
#x1 = pd.to_datetime(scenario_1.index)
x2=x1.map(dt.datetime.toordinal)#converts datetime datatype into a numerical value that can be applied in the regression
 
X = x2.values.reshape(-1, 1)  # values converts it into a numpy array
Y = y1.values.reshape(-1, 1)  # -1 means that calculate the dimension of rows, but have 1 column
 
linear_regressor = LinearRegression()  # create object for the class
linear_regressor.fit(X, Y)  # perform linear regression
Y_pred = linear_regressor.predict(X)  # make predictions
print(Y_pred)


#My regression, inspired by https://www.edvancer.in/step-step-guide-to-execute-linear-regression-python/
#x_reg = np.array(x2)
#y_reg = np.array(y1)
#
#ax = (((np.mean(x_reg)*np.mean(y_reg))-np.mean(x_reg*y_reg))/
#     ((np.mean(x_reg)*np.mean(x_reg))-np.mean(x_reg*x_reg)))
#ax = round(ax,2)
#b = (np.mean(y_reg) - np.mean(x_reg)*ax)
#b =round(b,2)
#print(ax)
#print(b)
#
#reg_line = [(ax*x_reg)+b for x_reg in x2]
#
#regressionlabel =('Linear regression: ' + str(ax)+'x + '+ str(b))
#std = np.std(reg_line)
##print('STD: ' + str(std))

#plt.scatter(X, Y)
#plt.plot(x2, reg_line, 'k-.', label ='Linear regression', linewidth=3)
plt.plot(X, Y_pred, 'k-.', label='Linear regression')
#plt.ylim(bottom=18, top=21.5) #Limits for y-axis UNIT 1
#plt.ylim(bottom=21, top=29.5) #Limits for y-axis UNIT 1
plt.legend()

##Text for Unit 1 mean of coolers
#plt.text(736420,18.7,'39,5 MW')#1 25032017
#plt.text(736430,18.92,'40,0 MW')#2 31032017
#plt.text(736435,19.2,'40,0 MW')#3 28062017
#plt.text(736540,19.3,'40,0 MW')#4 13102017
#plt.text(736681,18.87,'39,0 MW')#5 11122017
#plt.text(736805,19.45,'41,0 MW', horizontalalignment='right')#6 28042018
#plt.text(736940,20.05,'41,0 MW')#7 25082018
#plt.text(737042,19.16,'40,0 MW')#8 27022019
#plt.text(737211,19.55,'40,0 MW', horizontalalignment='right')#9 #08062019

##Text for Unit 2 mean of coolers
#plt.text(736470,25.8,'40,0 MW')#1 
#plt.text(736525,26.14,'40,0 MW')#2 
#plt.text(736613,26.28,'40,0 MW')#3 
#plt.text(736758,24.64,'39,0 MW')#4 
#plt.text(736880,25.43,'40,0 MW')#5 
#plt.text(737125,25.53,'39,7 MW')#6
#plt.text(737173,26.2,'40,0 MW', horizontalalignment='right')#7 
#plt.text(737225,27.08,'40,0 MW', horizontalalignment='right')#8


plt.show()
################################ stop time ####################################
t2 = time.perf_counter()
dt = t2-t1

#print('Processing ended   '+ time.asctime())
print('Calculating time: ' + str(dt) +' s')

warnings.simplefilter('error')
