# -*- coding: utf-8 -*-
"""
Created on Thu Jun 27 17:03:14 2019
@author: lauvl

PREREQUISITES:
    The CSV files that are exported from the analyzer tool must use ; as (field) separator and , as decimal separatator
    The exported files must include following signals:
        Generator_cold_air_1-4
        Generator_warm_air_1 and 2
        Power
        Setpoint_active_power
        

NOTES:
    The values in the exported CSV are stored in reversed chronological order, 
    meaning that the last valid value is placed first. 
    If the file is inspected in Excel it must then be edited to include the separator
    
SPECIFICS:
    This script is designed to handle the four cold air values
    combined with the two hot air measurements, as well as the power
    for each of the generators
"""
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import time
import pandas as pd
import warnings
from IPython import get_ipython
get_ipython().run_line_magic('matplotlib', 'qt')       #Shows plots in separate window
#get_ipython().run_line_magic('matplotlib', 'inline')  #Gives inline plots

#Cold and warm air
homePath = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Exported data/Scenario 1/Unit 1/'
#homePath = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Exported data/Scenario 1/Unit 2/'

#start time
t1 = time.perf_counter()
#print('Processing started '+ time.asctime())

# =============================================================================
# ##############################import data######################################
# #filename = 'Unit 1 scenario 1_25032017' # First
# #filename = 'Unit 1 scenario 1_11122017'
# #filename = 'Unit 1 scenario 1_25082018'
# filename = 'Unit 1 scenario 1_08062019' # Last
# 
# filetype = '.csv'
# path = homePath + filename + filetype         #Seprated into an extra variable to allow re-use of filename
# 
# df = pd.read_csv(path, sep=';', skiprows=1)   #Skips first row, because of meta- data(separator)
# #df = pd.read_csv(path, sep=';')              #Kun for test, må bruke import over pga meta-data
# 
# df = df.iloc[::-1].reset_index(drop=True)     #Reverses the order of the data to make it chronological and resets the index accordingly
# #print('df')
# #print(df)
# 
# ####  Resampling and combining measurements in one Data Frame  ################ 
# """
# The method creates a large resampled dataframe that consists of the 
# timestamps and measurements for each signal/measurement in the exported CSV-file
# 
# Prerequisites
# ----------
# CSV-file with one, or more, measurements.
# The CSV must be separated by a (;), and the delimitor must be(,).
# 
# Parameters
# ----------
# Dataframe containing imported data.
# 
# Returns
# ----------
# Pandas Dataframe
# """
# 
# noColumns = len(df.columns)                                                           #Finds the number of columns in the imported data stored in df
# noMeasurements = int(noColumns/7)                                                     #Finds the number of measurements. Each measurement contains 7 columns 
# 
# appended_data = []                                                                    #Empty list to store list of dataframes
# 
# j=0                                                                                   #Counter for column access
# #noMeasurements=1#FJERNES
# for i in range(noMeasurements):
#     datetime=pd.to_datetime(df.iloc[:,j]).dropna()                                    #Collects timestamp and converts it to python datetime-format and removes NaTs
# #    datetime=pd.to_datetime(df.iloc[:,0]).dropna()#Fjernes
# #    print('datetime')
# #    print(datetime)
#     measure_nan = df.iloc[:,j+5].dropna()                                            #Drops NaNs from the data frame
# #    measure_nan = df.iloc[:,5].dropna()#Fjernes 
# #    print('measure_nan')
# #    print(measure_nan)
#     measurement = measure_nan.apply(lambda x: x.replace(',', '.')).astype('float')    #Collects measurement, replaces , with ., in order to be able to convert it to a float
# #    print('measurement')
# #    print(measurement)
#     ts = pd.concat([datetime, measurement], axis=1)                                   #Concatenate corresponding timestamp and measurement
# #    print('ts')
# #    print(ts)
#     remove_duplicates = ts.drop_duplicates(subset=['Date ' + str(i+1)], keep='first') #Drops value when there are duplicate timestamps. Keeps the first value
# #    print('remove_duplicates')
# #    print(remove_duplicates)
# #    remove_duplicates.drop(remove_duplicates.head(2).index,inplace=True)              #The first TWO values often have a very large gap, to ensure that it is not given too much weight, it is dropped
# #    remove_duplicates.drop(remove_duplicates.tail(1).index,inplace=True)              #The last values often has a very large gap, to ensure that it is not given too much weight, it is dropped
#     reindex = remove_duplicates.set_index('Date ' + str(i+1))                         #Re-index to get datetime as index, necessary as index for resampling-method
# #    print('reindex')
# #    print(reindex)
#     #S is seconds, T is minute. '3T' is 3 minutes, H is hour, D is day
# #    resample = reindex.resample('S').pad()                                           #Second-sampling with forwardfill(resample)
# #    resample = reindex.resample('T').pad()                                           #Minute-sampling with forwardfill(resample)
# #    resample = reindex.resample('T').bfill() #Backward fill for filling of nans
# #    resample = reindex.resample('T').interpolate(method='values') #value interpolation for filling of nans
#     resample = reindex.resample('T').mean()
# #    resampled = resample.fillna(method='ffill')
#     resampled = resample.fillna(method='bfill')
# #    resampled = resample.interpolate(method='index')
# #    resampled = resample.interpolate(method='time')
# #    resampled = resample.interpolate(method='values')
# #    resample = reindex.resample('T').interpolate(method='time') #time interpolation for filling of nans
# #    resample = reindex.resample('H').pad()                                           #Hour-sampling with forwardfill(resample)
# #    resample = reindex.resample('D').pad()                                           #Day-sampling with forwardfill(resample)
#     resample_drop_first_row = resampled.drop(resampled.index[[0]])                      #If the value provided to the resampling method is second-sampled it will give a NaN on the first value, and be reflected on the second, now minute-sampled value
#     df_index = resample_drop_first_row.reset_index()                                  #Reindexes the dataframe to get it zero-indexed. This is necessary in order to have the same index for all dataframes, which is a requirement for appending
# #    print('df_index')
# #    print(df_index)
#     appended_data.append(df_index)                                                    #Appends the dataframe to a list of dataframes
#     j+=7                                                                              #Increments the counter with the number of colums per measurement, to gather all the measurements
# appended_data = pd.concat(appended_data, axis=1)                                      #Concatenate the dataframes to one large dataframe
# resampled_df = appended_data                                                          #Renames the large dataframe by assigning it to a new variable that better explains how the data has been preprocessed
# #print(resampled_df)
# 
# 
# #### Method for finding the first common timestamp in a dynamic dataframe #####
# noColumns_complete = len(resampled_df.columns)                               #Finds the number of columns in the imported data stored in df
# noMeasurements_complete = int(noColumns_complete/2)                                   #Finds the number of measurements. Each measurement(now) contains 2 columns; timestamp and measurement 
# 
# ###Method below finds the first common date in the first row, ergo the last starting date
# k=0
# dt1 = resampled_df.iloc[0,0]                                                          #First timestamp in first column
# for i in range(noMeasurements_complete-1):                                            #Does the operation for every measurement
#     dt2 = resampled_df.iloc[0, (k+2)]                                                 #Second timestamp, located in the third column
#     if dt1 < dt2:                                                                     #Compares the first and last timestamp to find the largest(most recent)
#         mostRecent = dt2                                                              #If dt2 is the largest, it is assigned to the mostRecent variable
#         dt1 = dt2                                                                     #dt2 is also assigned to the dt1 vairable to compare this value to the timestamp of the next measurement
#     else:                                                                             #if dt1 is more recent than dt2, the value of dt1 is kept as mostRecent timestamp
#         mostRecent = dt1
#     k += 2                                                                            #Iterate the K-variable to go to next datetime-column
# print('mostRecent')
# print(mostRecent)
#     
# ##Method below crops the dataframe 
# cropped_data = []                                                                                           #Empty list to store cropped dataframes
# m=0
# for l in range(noMeasurements_complete):                                                                    # Will perform an iteration for every measurement
#     res_index = next((i for i, j in enumerate(resampled_df.iloc[:, m]) if j == mostRecent), None)  #"Search" each column to find the index for the datetime stored in mostRecent-variable from previous method 
# #Method above is inspired by SilenGhost's reply on Stackoverflow, last visited 12.06.19(https://stackoverflow.com/questions/3229626/python-finding-index-of-first-non-empty-item-in-a-list/3229644)
#     datetime_crop = resampled_df.iloc[res_index:, m]                                               # Crops every datetime-column from the top down to the index provided by previous value which is the most recent common date
#     measurement_crop = resampled_df.iloc[res_index:, m+1]                                          # Crops every measurement-column from the top down to the index provided by previous value
#     ts_crop = pd.concat([datetime_crop, measurement_crop], axis=1)                                          #Concatenates the cropped datetime and measurement columns to a timeseries
#     ts_crop = ts_crop.reset_index(drop=True)                                                                #Resets the index for the timeseries, since it has been cropped top-down, since last index reset
#     cropped_data.append(ts_crop)                                                                            #Appends the cropped dataframe to a list of dataframes(together with the ones already cropped)
#     m+=2                                                                                                    #Increments the counter with the number of colums per measurement, to gather all the measurements
# cropped_data = pd.concat(cropped_data, axis=1)                                                              #Concatenate the dataframes to one large dataframe
# cropped_resampled_data = cropped_data.dropna()                                                                 #Drops any row that contains a NaN(at end of the complete dataframe(due to uneven number of samples for each measurement/uneven length of timeseries))
# 
# #print(cropped_resampled_data)
# 
# ################# Method for removing duplicate timestap columns #############
# reindexed_cropped_resampled_data = cropped_resampled_data.set_index(cropped_resampled_data.columns[0])                     #Set one common index for all measurement values
# reindexed_cropped_resampled_data.index.names = ['Timestamp']                                                     #Renames the common index column
# 
# #Finds the number of columns and timestamps for drop of dynamic number of columns
# noColumns_prd = len(reindexed_cropped_resampled_data.columns)                                                    #Finds the number of columns in the imported data stored in df
# noTimestamps_prd = int(noColumns_prd/2)                                                                     #Finds the number of measurements
#   
# #o = noTimestamps_prd+1                                                                                     #For addressing coloumn name in method below
# o = noColumns_prd-2                                                                                         #For addressing coloumn index in method below
# for n in range(noTimestamps_prd ):
# #    preprocessed_reindexed_data.drop(columns=["Date " + str(o)], axis = 1, inplace=True)                   #For addressing coloumn name 
#     reindexed_cropped_resampled_data.drop(reindexed_cropped_resampled_data.columns[o], axis = 1, inplace=True)        #For addressing coloumn index
# #    o -= 1                                                                                                 #For addressing coloumn name in method above
#     o -= 2   
#                                                                                                #For addressing coloumn index in method below
# df_preprocessed = reindexed_cropped_resampled_data #The preprocessed dataframe, renamed to emphasize that this is the finished dataframe
# #df_preprocessed.to_excel(filename + 'seconds_preprocessed.xlsx')                               #Writes the (often very) large dataframe to an Excel-file(Very time consuming)
# #print(df_preprocessed)
# #print(df_preprocessed.dtypes)
# 
# ########################Analysis ##############################################
# 
# #Lage ein gjennomsnitt av alle verdier på ein rad
# 
# df_preprocessed['Mean_cold'] = df_preprocessed.iloc[:,0:4].mean(axis=1)
# df_preprocessed['Mean_warm'] = df_preprocessed.iloc[:,4:6].mean(axis=1)
# df_preprocessed['Mean_diff'] = df_preprocessed.iloc[:,9]-df_preprocessed.iloc[:,8]
# df_preprocessed['Cooler 1'] = df_preprocessed.iloc[:,9]-df_preprocessed.iloc[:,0]
# df_preprocessed['Cooler 2'] = df_preprocessed.iloc[:,9]-df_preprocessed.iloc[:,1]
# df_preprocessed['Cooler 3'] = df_preprocessed.iloc[:,9]-df_preprocessed.iloc[:,2]
# df_preprocessed['Cooler 4'] = df_preprocessed.iloc[:,9]-df_preprocessed.iloc[:,3]
# df_preprocessed['Mean, coolers'] = df_preprocessed.iloc[:,11:15].mean(axis=1)
# print(df_preprocessed.dtypes)
# print('\nFirst valid index: '+ str(df_preprocessed.first_valid_index()))
# print('Last valid index: '+ str(df_preprocessed.last_valid_index()))
# 
# #print(df_preprocessed[['Mean_cold', 'Mean_warm', 'Mean_diff']])
# #print(df_preprocessed[['Cooler 1', 'Cooler 2', 'Cooler 3', 'Cooler 4']])
# 
# df_preprocessed.to_excel('bfill.xlsx', sheet_name=filename) 
# =============================================================================

#######################################  Plot  ##############################
#Henter inn igjen den genererte fila og plotter for minutt og sekung
plt.rcParams.update({'font.size': 20})

#filename = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Scripts/Sampling_sek1.xlsx'
#filename2 = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Scripts/Sampling_min1.xlsx'
#
##filename = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Scripts/Sampling_sek2.xlsx'
##filename2 = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Scripts/Sampling_min2.xlsx'
#
##filename = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Scripts/Sampling_sek3.xlsx'
##filename2 = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Scripts/Sampling_min3.xlsx'
#
##filename = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Scripts/Sampling_sek4.xlsx'
##filename2 = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Scripts/Sampling_min4.xlsx'
#
#df = pd.read_excel(filename)
#df2 = pd.read_excel(filename2)
#
#datetime1 = pd.to_datetime(df.iloc[:,0])
#measure1 = df.iloc[:,7]
#ts1 = pd.concat([datetime1, measure1], axis=1)
#ferdig1 = ts1.set_index('Timestamp')
#
#datetime2 = pd.to_datetime(df2.iloc[:,0])
#measure2 = df2.iloc[:,7]
#ts2 = pd.concat([datetime2, measure2], axis=1)
#ferdig2 = ts2.set_index('Timestamp')


# =============================================================================
# #Plot to compare minute and second sampling
# fig = plt.figure()
# fx1=fig.add_subplot(111)
# fx1.plot(ferdig1, '--', fillstyle='none', label='Second sampling')#linewidth=3, markersize=15,
# fx1.plot(ferdig2, '--x', fillstyle='none', markersize=10, linewidth=3, label='Minute sampling')
# fx1.legend()
# fx1.set_xlabel('Time')
# fx1.set_ylabel('Power [MW]')
# #fx1.set_ylabel('Temperature [K]')
# 
# monthyearFmt = mdates.DateFormatter('%H:%M')
# fx1.xaxis.set_major_formatter(monthyearFmt)
# =============================================================================

#Gathers files for a comparison of filling techniques

filename1 = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Scripts/pad.xlsx'
filename2 = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Scripts/bfill.xlsx'
#filename3 = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Scripts/value_interpolation.xlsx'
filename3 = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Scripts/index_interpolation.xlsx'
filename4 = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Scripts/time_interpolation.xlsx'
    
df1 = pd.read_excel(filename1)
df2 = pd.read_excel(filename2)
df3 = pd.read_excel(filename3)
df4 = pd.read_excel(filename4)

datetime1 = pd.to_datetime(df1.iloc[:,0])
measure1 = df1.iloc[:,7]
ts1 = pd.concat([datetime1, measure1], axis=1)
ferdig1 = ts1.set_index('Timestamp')

datetime2 = pd.to_datetime(df2.iloc[:,0])
measure2 = df2.iloc[:,7]
ts2 = pd.concat([datetime2, measure2], axis=1)
ferdig2 = ts2.set_index('Timestamp')

datetime3 = pd.to_datetime(df3.iloc[:,0])
measure3 = df3.iloc[:,7]
ts3 = pd.concat([datetime3, measure3], axis=1)
ferdig3 = ts3.set_index('Timestamp')

datetime4 = pd.to_datetime(df4.iloc[:,0])
measure4 = df4.iloc[:,7]
ts4 = pd.concat([datetime4, measure4], axis=1)
ferdig4 = ts4.set_index('Timestamp')



#Plot to compare filling techniques
fig = plt.figure()#Plot for both graphs

#Plots first graph
fx1=fig.add_subplot(411)
fx1.plot(ferdig1, '--o', fillstyle='none', label='Forward fill')
fx1.legend()

#Plots second graph
fx2=fig.add_subplot(412)
fx2.plot(ferdig2, '--o', fillstyle='none', label='Backward fill')
fx2.legend()

#Plots third graph
fx3=fig.add_subplot(413)
fx3.plot(ferdig3, '--o', fillstyle='none', label='Value interpolation')
fx3.legend()

#Plots fourth graph
fx4=fig.add_subplot(414)
fx4.plot(ferdig4, '--o', fillstyle='none', label='Time interpolation')
fx4.legend()

fx1.set_xlabel('Time')
fx1.set_ylabel('Power [MW]')
fx2.set_ylabel('Power [MW]')
fx3.set_ylabel('Power [MW]')
fx4.set_ylabel('Power [MW]')
#fx1.set_ylabel('Temperature [K]')

monthyearFmt = mdates.DateFormatter('%H:%M')
fx1.xaxis.set_major_formatter(monthyearFmt)
fx4.xaxis.set_major_formatter(monthyearFmt)

################################ stop time ####################################
t2 = time.perf_counter()
dt = t2-t1

#print('Processing ended   '+ time.asctime())
print('Calculating time: ' + str(dt) +' s')

warnings.simplefilter('error')