# -*- coding: utf-8 -*-
"""
Created on Fri May 24 10:13:17 2019

@author: lauvlandm

PREREQUISITES:
    The CSV files that are exported from the analyzer tool must use ; as (field) separator and , as decimal separatator

NOTES:
    The values in the exported CSV are stored in reversed chronological order, 
    meaning that the last valid value is placed first.
    
SPECIFICS:
    This script is designed to handle the four cold air measurements that belong to each generator
"""
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import time
import pandas as pd
import warnings
from IPython import get_ipython
get_ipython().run_line_magic('matplotlib', 'qt')       #Shows plots in separate window
#get_ipython().run_line_magic('matplotlib', 'inline')  #Gives inline plots

#Cold air
homePath = 'C:/Users/lauvl/Google Drive/Bachelor 2019/Analysis and modelling/Exported data/Generator/Cold_air/'

#start time
t1 = time.perf_counter()
#print('Processing started '+ time.asctime())

##############################import data######################################
#filename = 'Cold air unit 1_05-23-2019_14-20_old'
#filename ='Cold air unit 1_05-23-2019_14-20'
#filename = 'Cold air unit 1_05-23-2019_14-20 - Copy'#Test file with 10 values
#filename = 'Cold air unit 1_05-23-2019_16-17' #
filename = 'Cold air unit 1_05-23-2019_16-17_crop' #Cropped for better view

filetype = '.csv'
path = homePath + filename + filetype         #Seprated into an extra variable to allow re-use of filename

df = pd.read_csv(path, sep=';', skiprows=1)   #Skips first row, because of meta- data(separator)
#df = pd.read_csv(path, sep=';')              #Kun for test, må bruke import over pga meta-data

df = df.iloc[::-1].reset_index(drop=True)     #Reverses the order of the data to make it chronological and resets the index accordingly

####  Resampling and combining measurements in one Data Frame  ################ 
"""
The method creates a large resampled dataframe that consists of the 
timestamps and measurements for each signal/measurement in the exported CSV-file

Prerequisites
----------
CSV-file with one, or more, measurements.
The CSV must be separated by a (;), and the delimitor must be(,).

Parameters
----------
Dataframe containing imported data.

Returns
----------
Pandas Dataframe
"""

noColumns = len(df.columns)                                                           #Finds the number of columns in the imported data stored in df
noMeasurements = int(noColumns/7)                                                     #Finds the number of measurements. Each measurement contains 7 columns 

appended_data = []                                                                    #Empty list to store list of dataframes

j=0                                                                                   #Counter for column access
for i in range(noMeasurements):
    datetime=pd.to_datetime(df.iloc[:,j]).dropna()                                    #Collects timestamp and converts it to python datetime-format and removes NaTs
    measure_nan = df.iloc[:,j+5].dropna()                                             #Drops NaNs from the data frame
    measurement = measure_nan.apply(lambda x: x.replace(',', '.')).astype('float')    #Collects measurement, replaces , with ., in order to be able to convert it to a float
    ts = pd.concat([datetime, measurement], axis=1)                                   #Concatenate corresponding timestamp and measurement
    remove_duplicates = ts.drop_duplicates(subset=['Date ' + str(i+1)], keep='first') #Drops value when there are duplicate timestamps. Keeps the first value
    drop_first_row = remove_duplicates.drop(remove_duplicates.index[[0]])             #The first value often has a very large gap, to ensure that it is not given too much weight it is dropped
    reindex = drop_first_row.set_index('Date ' + str(i+1))                            #Re-index to get datetime as index, necessary as index for resampling-method
    #S is seconds, T is minute. '3T' is 3 minutes, H is hour, D is day
#    resample = reindex.resample('S').pad()                                           #Second-sampling with forwardfill(resample)
    resample = reindex.resample('T').pad()                                           #Minute-sampling with forwardfill(resample)
#    resample = reindex.resample('H').pad()                                           #Hour-sampling with forwardfill(resample)
#    resample = reindex.resample('D').pad()                                           #Day-sampling with forwardfill(resample)
    resample_drop_first_row = resample.drop(resample.index[[0]])                      #If the value provided to the resampling method is second-sampled it will give a NaN on the first value, and be reflected on the second, now minute-sampled value
    df_index = resample_drop_first_row.reset_index()                                  #Reindexes the dataframe to get it zero-indexed. This is necessary in order to have the same index for all dataframes, which is a requirement for appending
    appended_data.append(df_index)                                                    #Appends the dataframe to a list of dataframes
    j+=7                                                                              #Increments the counter with the number of colums per measurement, to gather all the measurements
appended_data = pd.concat(appended_data, axis=1)                                      #Concatenate the dataframes to one large dataframe
resampled_df = appended_data                                                          #Renames the large dataframe by assigning it to a new variable that better explains how the data has been preprocessed

#### Method for finding the first common timestamp in a dynamic dataframe #####
noColumns_complete = len(resampled_df.columns)                               #Finds the number of columns in the imported data stored in df
noMeasurements_complete = int(noColumns_complete/2)                                   #Finds the number of measurements. Each measurement(now) contains 2 columns; timestamp and measurement 

###Method below finds the first common date in the first row, ergo the last starting date
k=0
dt1 = resampled_df.iloc[0,0]                                                          #First timestamp in first column
for i in range(noMeasurements_complete-1):                                            #Does the operation for every measurement
    dt2 = resampled_df.iloc[0, (k+2)]                                                 #Second timestamp, located in the third column
    if dt1 < dt2:                                                                     #Compares the first and last timestamp to find the largest(most recent)
        mostRecent = dt2                                                              #If dt2 is the largest, it is assigned to the mostRecent variable
        dt1 = dt2                                                                     #dt2 is also assigned to the dt1 vairable to compare this value to the timestamp of the next measurement
    else:                                                                             #if dt1 is more recent than dt2, the value of dt1 is kept as mostRecent timestamp
        mostRecent = dt1
    k += 2                                                                            #Iterate the K-variable to go to next datetime-column
    
##Method below crops the dataframe 
cropped_data = []                                                                                           #Empty list to store cropped dataframes
m=0
for l in range(noMeasurements_complete):                                                                    # Will perform an iteration for every measurement
    res_index = next((i for i, j in enumerate(resampled_df.iloc[:, m]) if j == mostRecent), None)  #"Search" each column to find the index for the datetime stored in mostRecent-variable from previous method 
#Method above is inspired by SilenGhost's reply on Stackoverflow, last visited 12.06.19(https://stackoverflow.com/questions/3229626/python-finding-index-of-first-non-empty-item-in-a-list/3229644)
    datetime_crop = resampled_df.iloc[res_index:, m]                                               # Crops every datetime-column from the top down to the index provided by previous value which is the most recent common date
    measurement_crop = resampled_df.iloc[res_index:, m+1]                                          # Crops every measurement-column from the top down to the index provided by previous value
    ts_crop = pd.concat([datetime_crop, measurement_crop], axis=1)                                          #Concatenates the cropped datetime and measurement columns to a timeseries
    ts_crop = ts_crop.reset_index(drop=True)                                                                #Resets the index for the timeseries, since it has been cropped top-down, since last index reset
    cropped_data.append(ts_crop)                                                                            #Appends the cropped dataframe to a list of dataframes(together with the ones already cropped)
    m+=2                                                                                                    #Increments the counter with the number of colums per measurement, to gather all the measurements
cropped_data = pd.concat(cropped_data, axis=1)                                                              #Concatenate the dataframes to one large dataframe
cropped_resampled_data = cropped_data.dropna()                                                                   #Drops any row that contains a NaN(at end of the complete dataframe(due to uneven number of samples for each measurement/uneven length of timeseries))

################# Method for removing duplicate timestap columns #############
reindexed_cropped_resampled_data = cropped_resampled_data.set_index(cropped_resampled_data.columns[0])                     #Set one common index for all measurement values
reindexed_cropped_resampled_data.index.names = ['Timestamp']                                                     #Renames the common index column

#Finds the number of columns and timestamps for drop of dynamic number of columns
noColumns_prd = len(reindexed_cropped_resampled_data.columns)                                                    #Finds the number of columns in the imported data stored in df
noTimestamps_prd = int(noColumns_prd/2)                                                                     #Finds the number of measurements
  
#o = noTimestamps_prd+1                                                                                     #For addressing coloumn name in method below
o = noColumns_prd-2                                                                                         #For addressing coloumn index in method below
for n in range(noTimestamps_prd ):
#    preprocessed_reindexed_data.drop(columns=["Date " + str(o)], axis = 1, inplace=True)                   #For addressing coloumn name 
    reindexed_cropped_resampled_data.drop(reindexed_cropped_resampled_data.columns[o], axis = 1, inplace=True)        #For addressing coloumn index
#    o -= 1                                                                                                 #For addressing coloumn name in method above
    o -= 2   
                                                                                               #For addressing coloumn index in method below
df_preprocessed = reindexed_cropped_resampled_data #The preprocessed dataframe, renamed to emphasize that this is the finished dataframe
#df_preprocessed.to_excel(filename + 'seconds_preprocessed.xlsx')                               #Writes the (often very) large dataframe to an Excel-file(Very time consuming)

########################Analysis ##############################################

#Mean for all temperatures by row

df_preprocessed['Mean'] = df_preprocessed.mean(axis=1)
df_preprocessed['Mean+1%'] = df_preprocessed.iloc[:,4]*1.01
df_preprocessed['Mean-1%'] = df_preprocessed.iloc[:,4]*0.99
df_preprocessed['St.dev.'] = df_preprocessed.iloc[:,4].std(axis=0)
df_preprocessed['St.dev.-'] = df_preprocessed.iloc[:,4] - (df_preprocessed.iloc[:,7]*2)
df_preprocessed['St.dev.+'] = df_preprocessed.iloc[:,4] + (df_preprocessed.iloc[:,7]*2)

print('df_preprocessed[St.dev.]')
print(df_preprocessed['St.dev.'])


#df_preprocessed.to_excel(filename + 'med bånd.xlsx') 

########################################  Plot  ##############################
#plt.plot(df_preprocessed)

################# Fire plott i "vindu" for å illustrere alarmgrenser??
fig, axes = plt.subplots(2,2, sharex=True, sharey=True)

x = df_preprocessed.index
y1 = df_preprocessed[['Measurement 1']]
y2 = df_preprocessed[['Measurement 2']]
y3 = df_preprocessed[['Measurement 3']]
y4 = df_preprocessed[['Measurement 4']]
y5 = df_preprocessed['St.dev.-']
y6 = df_preprocessed['St.dev.+']
y7 = df_preprocessed['Mean+1%']
y8 = df_preprocessed['Mean-1%']


# one plot on each subplot
Cold_air_1, = axes[0][0].plot(x,y1, 'C1', label='Cold air 1')
axes[0][0].plot(x,y5, 'k:', label='Lower threshold value', linewidth=3)
axes[0][0].plot(x,y6, 'C3:', label='Upper threshold value',linewidth=3)
#axes[0][0].plot(x,y7, 'k:', label='Lower threshold value', linewidth=3)
#axes[0][0].plot(x,y8, 'C3:', label='Upper threshold value',linewidth=3)

Cold_air_2, = axes[0][1].plot(x,y2, 'C2', label='Cold air 2')
axes[0][1].plot(x,y5, 'k:', label='Lower threshold value', linewidth=3)
axes[0][1].plot(x,y6, 'C3:', label='Upper threshold value',linewidth=3)
#axes[0][1].plot(x,y7, 'k:', label='Lower threshold value', linewidth=3)
#axes[0][1].plot(x,y8, 'C3:', label='Upper threshold value',linewidth=3)

Cold_air_3, = axes[1][0].plot(x,y3, 'C4', label='Cold air 3')
axes[1][0].plot(x,y5, 'k:', label='Lower threshold value', linewidth=3)
axes[1][0].plot(x,y6, 'C3:', label='Upper threshold value',linewidth=3)
#axes[1][0].plot(x,y7, 'k:', label='Lower threshold value', linewidth=3)
#axes[1][0].plot(x,y8, 'C3:', label='Upper threshold value',linewidth=3)
axes[1][0].set_xlabel('Time')

Cold_air_4, = axes[1][1].plot(x,y4, 'C5', label='Cold air 4')
threshold_1, = axes[1][1].plot(x,y5, 'k:', label='Lower threshold value', linewidth=3)
threshold_2, = axes[1][1].plot(x,y6, 'C3:', label='Upper threshold value',linewidth=3)
#threshold_1, = axes[1][1].plot(x,y7, 'k:', label='Lower threshold value', linewidth=3)
#threshold_2, = axes[1][1].plot(x,y8, 'C3:', label='Upper threshold value',linewidth=3)
axes[1][1].set_xlabel('Time')

plt.xticks([])
#axes[0][1].set_yticks([])
#axes[1][1].set_yticks([])
axes[0][0].legend(handles=[Cold_air_1], loc=1)
axes[0][1].legend(handles=[Cold_air_2], loc=2)
axes[1][0].legend(handles=[Cold_air_3], loc=1)
axes[1][1].legend(handles=[Cold_air_4], loc=2)

fig.legend(handles=[threshold_1, threshold_2], loc=1)
plt.rcParams.update({'font.size': 20})

#fig.legend()
plt.show()

# =============================================================================
# ################# To plott for å illustrere alarmgrenser
# plt.rcParams.update({'font.size': 20})
# monthyearFmt = mdates.DateFormatter('%d %b')
# 
# fig = plt.figure()#Plot for both graphs
# #fig, (ax1, ax2) = plt.subplots(2, sharex=True)#, constrained_layout=True)
# fig.subplots_adjust(hspace=0)
# #Plots first graph
# fx1=fig.add_subplot(211)
# plt.ylabel('Temperature[K]')
# plt.xlabel('Time')
# plt.xticks(rotation=25)
# #fx1.plot(df_preprocessed.iloc[:,0:4])
# fx1.plot(df_preprocessed[['Measurement 1']], 'C1', label='Cold air 1')
# fx1.plot(df_preprocessed[['Measurement 2']],'C2', label='Cold air 2')
# fx1.plot(df_preprocessed[['Measurement 3']],'C4', label='Cold air 3')
# fx1.plot(df_preprocessed[['Measurement 4']],'C5', label='Cold air 4')
# plt.legend(loc=1)
# 
# 
# #Plots second graph
# fx2=fig.add_subplot(212)
# plt.ylabel('Temperature[K]')
# #plt.xlabel('Time')
# fx2.xaxis.set_major_formatter(monthyearFmt)
# plt.xticks(rotation=30)
# #fx2.plot(df_preprocessed.iloc[:,4],'g')
# #fx2.plot(df_preprocessed.iloc[:,5:7],'r:')
# fx2.plot(df_preprocessed.iloc[:,4],'C9', label='Cold air, mean')
# #fx2.plot(df_preprocessed['Mean+1%'],'C3:', label='Upper threshold value', linewidth=3)
# #fx2.plot(df_preprocessed['Mean-1%'],'k:', label='Lower threshold value', linewidth=3)
# # fx2.plot(df_preprocessed['St.dev.'], label='Standard deviation', linewidth=3)
# fx2.plot(df_preprocessed['St.dev.+'], 'C3:', label='Upper threshold value', linewidth=3)
# fx2.plot(df_preprocessed['St.dev.-'], 'k:', label='Lower threshold value', linewidth=3)
# plt.legend(loc=1)
# =============================================================================

# =============================================================================
# ################# Tre plott for å illustrere alarmgrenser
# plt.rcParams.update({'font.size': 20})
# monthyearFmt = mdates.DateFormatter('%d %b')
# 
# fig = plt.figure()#Plot for all graphs
# fig.subplots_adjust(hspace=0)
# 
# #Plots first graph
# fx1=fig.add_subplot(311)
# plt.ylim(bottom=284.5, top=295.5)
# plt.ylabel('Temperature[K]')
# plt.xlabel('Time')
# plt.xticks(rotation=25)
# fx1.plot(df_preprocessed[['Measurement 1']], 'C1', label='Cold air 1')
# fx1.plot(df_preprocessed[['Measurement 2']],'C2', label='Cold air 2')
# fx1.plot(df_preprocessed[['Measurement 3']],'C4', label='Cold air 3')
# fx1.plot(df_preprocessed[['Measurement 4']],'C5', label='Cold air 4')
# plt.legend()
# 
# 
# #Plots second graph
# fx2=fig.add_subplot(312)
# plt.ylim(bottom=284.5, top=295.5)
# plt.ylabel('Temperature[K]')
# plt.xlabel('Time')
# fx2.plot(df_preprocessed.iloc[:,4],'C9', label='Cold air, mean')
# fx2.plot(df_preprocessed['Mean+1%'],'C3:', label='Upper threshold value', linewidth=3)
# fx2.plot(df_preprocessed['Mean-1%'],'k:', label='Lower threshold value', linewidth=3)
# #fx2.plot(df_preprocessed['St.dev.'], label='Standard deviation', linewidth=3)
# #fx2.plot(df_preprocessed['St.dev.+'], 'C3:', label='Upper threshold value', linewidth=3)
# #fx2.plot(df_preprocessed['St.dev.-'], 'k:', label='Lower threshold value', linewidth=3)
# plt.legend(loc=1)
# 
# #Plots third graph
# fx2=fig.add_subplot(313)
# plt.ylim(bottom=284.5, top=295.5)
# plt.ylabel('Temperature[K]')
# #plt.xlabel('Time')
# fx2.xaxis.set_major_formatter(monthyearFmt)
# plt.xticks(rotation=30)
# fx2.plot(df_preprocessed[['Measurement 1']],'C1', label='Cold air 1')
# fx2.plot(df_preprocessed[['Measurement 2']],'C2', label='Cold air 2')
# fx2.plot(df_preprocessed[['Measurement 3']],'C4', label='Cold air 3')
# fx2.plot(df_preprocessed[['Measurement 4']],'C5', label='Cold air 4')
# #fx2.plot(df_preprocessed['St.dev.-'], 'k:', label='Lower threshold value', linewidth=3)
# #fx2.plot(df_preprocessed['St.dev.+'], 'C3:', label='Upper threshold value', linewidth=3)
# fx2.plot(df_preprocessed['Mean+1%'],'C3:', label='Upper threshold value', linewidth=3)
# fx2.plot(df_preprocessed['Mean-1%'],'k:', label='Lower threshold value', linewidth=3)
# #plt.legend()
# 
# =============================================================================
# =============================================================================
# 
# ################# Eitt plott for å illustrere alarmgrenser
plt.rcParams.update({'font.size': 20})
# #monthyearFmt = mdates.DateFormatter('%d %b')
# #Bare siste figur
# #plt.xaxis.set_major_formatter(monthyearFmt)
# plt.xticks(rotation=30)
# 
# plt.plot(df_preprocessed[['Measurement 1']], 'C1', label='Cold air 1')
# plt.plot(df_preprocessed[['Measurement 2']],'C2', label='Cold air 2')
# plt.plot(df_preprocessed[['Measurement 3']],'C4', label='Cold air 3')
# plt.plot(df_preprocessed[['Measurement 4']],'C5', label='Cold air 4')
# #plt.plot(df_preprocessed[['Mean']], 'C8-.', label='Mean', linewidth=5)
# plt.plot(df_preprocessed['St.dev.-'], 'k:', label='Lower threshold value', linewidth=3)
# plt.plot(df_preprocessed['St.dev.+'], 'C3:', label='Upper threshold value', linewidth=3)
# #plt.plot(df_preprocessed['Mean+1%'],'C3:', label='Upper threshold value', linewidth=3)
# #plt.plot(df_preprocessed['Mean-1%'],'k:', label='Lower threshold value', linewidth=3)
# plt.legend(loc=1)
# =============================================================================

################################ stop time ####################################
t2 = time.perf_counter()
dt = t2-t1

#print('Processing ended   '+ time.asctime())
print('Calculating time: ' + str(dt) +' s')

warnings.simplefilter('error')